/* VzImagination (C) 2018-2019 Stephane Charette <stephanecharette@gmail.com>
 * $Id: TestMatManipCompareExact.cpp 2741 2019-04-09 17:04:20Z stephane $
 */

#include "TestUtil.hpp"


TEST(CompareExact, Empty)
{
	cv::Mat mat1;
	cv::Mat mat2;

	ASSERT_TRUE(mat1.empty());
	ASSERT_TRUE(mat2.empty());
	ASSERT_TRUE(vz::compare_exact(mat1, mat2));

	cv::Mat mat3(5, 5, CV_8UC3, cv::Scalar(255, 255, 255)); // 5x5 white
	ASSERT_FALSE(vz::compare_exact(mat1, mat3));
}


TEST(CompareExact, DoesNotCompare)
{
	std::vector<cv::Mat> v;

	// none of these images should compare the same
	for (const auto & [path, name] : test_images())
	{
		cv::Mat mat = cv::imread(path);
		ASSERT_FALSE(mat.empty());

		// compare against all the other images we've loaded so far
		std::cout << "comparing " << name << " against " << v.size() << " other images" << std::endl;
		for (auto img : v)
		{
			ASSERT_FALSE(vz::compare_exact(mat, img));
		}

		v.push_back(mat);
	}
}


TEST(CompareExact, Greyscale1)
{
	for (const auto & [path, name] : test_images())
	{
		cv::Mat src = cv::imread(path, cv::IMREAD_GRAYSCALE);
		ASSERT_EQ(src.channels(), 1);

		cv::Mat dst = vz::to_greyscale(src);
		ASSERT_EQ(dst.channels(), 1);

		ASSERT_TRUE(vz::compare_exact(src, dst));
	}
}


TEST(CompareExact, Greyscale2)
{
	// I initially thought that r.jpg, g.jpg, and b.jpg would all match once converted to greyscale,
	// but it looks like different weights are assigned to each channel, so they don't convert to
	// identical shades of grey.  Thus, compare_exact() will return false.

	std::vector<cv::Mat> v;
	for (const auto & [path, name] : test_images())
	{
		cv::Mat mat1 = cv::imread(path);
		cv::Mat mat2 = vz::to_greyscale(mat1);

		ASSERT_EQ(mat1.channels(), 3);
		ASSERT_EQ(mat2.channels(), 1);

		v.push_back(mat1);
		v.push_back(mat2);
	}

	for (cv::Mat lhs : v)
	{
		// this image ("lhs") should match exactly 1 image in the entire vector of images (itself)
		size_t matches_found = 0;
		for (cv::Mat rhs : v)
		{
			if (vz::compare_exact(lhs, rhs))
			{
				matches_found ++;
			}
		}
		ASSERT_EQ(matches_found, 1);
	}
}
